/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact:  Sudipto Bal <sudipto.bal@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <pkgmgr-info.h>
#include <aul.h>
#include <bluetooth.h>
#include <bluetooth_internal.h>

#include "bm_bt_plugin.h"
#include "bm_bt_util.h"
#include "bm_log.h"

#define DEFAULT_APPID "Tizen"

/*
 * To get AppID from PID - Needs to be handled by the respective framework owners:
 * The PID might become outdated by the time data is read, which will create problems in mapping the application.
 * Sample API's and fucntion is provided below to get app-id based on available pid.
 * char *strAppId = NULL;
 * pid_t nProcessId = getpid();
 * app_manager_get_app_id(nProcessId, &strAppId);
 */
static char* get_current_appid(int pid, uid_t uid)
{
	ENTER;
	_I("getting caller appid with pid=[%d], uid=[%d]", pid, uid);

	int ret = 0;
	char appid[128] = { 0, };
	char *appid_ret = NULL;

	ret = aul_app_get_appid_bypid_for_uid(pid, appid, sizeof(appid), uid);

	if (ret < 0) {
		_E("fail to get current appid ret=[%d]\n", ret);
		appid_ret = strdup(DEFAULT_APPID);
	} else {
		appid_ret = strdup(appid);
	}

	if (appid_ret == NULL)
		_E("Memory Allocation Failed");

	EXIT;
	return appid_ret;
}

int init()
{
	ENTER;

/*	Initialize the necessary paramters and callbacks if required */
	bt_initialize();
	bt_adapter_init_battery_monitor(NULL, NULL);
	EXIT;
	return BM_PLUGIN_ERROR_NONE;
}

int deinit()
{
	ENTER;

/*	De-initialize callbacks & parameters */

	EXIT;
	return BM_PLUGIN_ERROR_NONE;
}

static void free_app_list(GSList *app_list)
{
	GSList *l = NULL;
	app_time_map_st2 *app_info;
	for (l = app_list; l != NULL; l = g_slist_next(l)){
		app_info = (app_time_map_st2 *)(l->data);
		if (app_info != NULL)
			free(app_info->app_id);
	}
	g_slist_free(app_list);
}

int get_feature_data(bm_data_h *handle, bm_plugin_data_type_e type)
{
	ENTER;

	bm_bluetooth_st *bluetooth_record_head = NULL;
	bm_bluetooth_st *bluetooth_record = NULL;
	app_time_map_st2 *app_record = NULL;

	bluetooth_record = (bm_bluetooth_st *)calloc(1, sizeof(bm_bluetooth_st));
	if (bluetooth_record == NULL) {
		_E("Calloc Failed");
		return BM_PLUGIN_ERROR_OUT_OF_MEMORY;
	}

	/* Fill bluetooth record with the values of the bm_bluetooth_st */
	bt_battery_info_s battery_data;
	memset(&battery_data, 0, sizeof(battery_data));
	int ret = bt_adapter_read_battery_info(&battery_data);

	if (ret != 0)
	{
		_E("battery info not read");
		free(bluetooth_record);
		return BM_PLUGIN_ERROR_NO_DATA;
	}

	bluetooth_record->startTime = battery_data.session_start_time;
	bluetooth_record->stopTime = battery_data.session_end_time;
	bluetooth_record->scanTime = battery_data.session_scan_time;
	bluetooth_record->connectedTime = battery_data.session_connected_time;

	_I("BT usage data for battery monitor");
	_I("startTime: %ld", bluetooth_record->startTime);
	_I("stopTime: %ld", bluetooth_record->stopTime);
	_I("scanTime: %u", bluetooth_record->scanTime);
	_I("connectedTime: %u", bluetooth_record->connectedTime);

	/* Getting app data */
	GSList *l = NULL;
	bt_battery_app_info_s *app_info = NULL;

	for (l = battery_data.atm_list; l != NULL; l = g_slist_next(l)) {

		app_record = (app_time_map_st2 *)calloc(1, sizeof(app_time_map_st2));
		if (app_record == NULL) {
			_E("Calloc Failed");
			free_app_list(bluetooth_record->atm_list);
			free(bluetooth_record);
			return BM_PLUGIN_ERROR_OUT_OF_MEMORY;
		}

		app_info = (bt_battery_app_info_s *)(l->data);
		if (app_info == NULL) {
			_E("app_info is NULL");
			free_app_list(bluetooth_record->atm_list);
			free(bluetooth_record);
			free(app_record);
			return BM_PLUGIN_ERROR_NO_DATA;
		}

		/* populate application record here with usage values */
		app_record->app_id = get_current_appid(app_info->pid, app_info->uid);
		app_record->time = app_info->time;
		app_record->rx = app_info->rx_bytes;
		app_record->tx = app_info->tx_bytes;

		/* Fill the app_record with the values of the app_time_map_st2 */
		bluetooth_record->atm_list =
			g_slist_append(bluetooth_record->atm_list, app_record);
	}

	/*Append BT data to head node as per design doc*/
	bluetooth_record_head = (bm_bluetooth_st *)calloc(1, sizeof(bm_bluetooth_st));
	if (bluetooth_record_head == NULL) {
		_E("Calloc Failed");
		free_app_list(bluetooth_record->atm_list);
		free(bluetooth_record);
		return BM_PLUGIN_ERROR_OUT_OF_MEMORY;
	}
	bluetooth_record_head->bt_data_list = g_slist_append(
				bluetooth_record_head->bt_data_list, bluetooth_record);
	*handle = bluetooth_record_head;

	EXIT;
	return BM_PLUGIN_ERROR_NONE;
}

static bm_api_st bluetooth_api = {
	.init = init,
	.deinit = deinit,
	.get_feature_data = get_feature_data
};

int plugin_init(bm_api_st **api)
{
	ENTER;

	*api = &bluetooth_api;

	EXIT;
	return 0;
}

int plugin_deinit(void)
{
	ENTER;

	EXIT;
	return 0;
}

BM_PLUGIN_ADD(BM_PLUGIN_ID_BLE, NAME, AUTHOR, VERSION, plugin_init, plugin_deinit);
