/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <pkgmgr-info.h>
#include <aul.h>
#include <gio/gio.h>
#include <glib-object.h>

#include "bm_dn_plugin.h"
#include "bm_dn_util.h"
#include "bm_log.h"

#define NETCONFIG_SERVICE				"net.netconfig"
#define NETCONFIG_NETWORK_PATH			"/net/netconfig/network"
#define NETCONFIG_BATTERY_INTERFACE		"net.netconfig.battery"

#define BATTERY_ATM_LIST            "atm_list"
#define BATTERY_ATM_APP_ID          "atm_app_id"
#define BATTERY_ATM_RX              "atm_rx"
#define BATTERY_ATM_TX              "atm_tx"

#define BATTERY_DN_TIME_LEVEL_0     "time_level_0"
#define BATTERY_DN_TIME_LEVEL_1     "time_level_1"
#define BATTERY_DN_TIME_LEVEL_2     "time_level_2"
#define BATTERY_DN_TIME_LEVEL_3     "time_level_3"
#define BATTERY_DN_TIME_LEVEL_4     "time_level_4"
#define BATTERY_DN_TIME_LEVEL_5     "time_level_5"
#define BATTERY_DN_TIME_LEVEL_6     "time_level_6"

#define BM_DN_DBUS_REPLY_TIMEOUT        (15 * 1000)

typedef void (*bm_dbus_dict_cb) (const char *key, GVariant *value, void *user_data);

static GVariant *__invoke_dbus_method(const char *dest, const char *path,
		const char *interface_name, const char *method, GVariant *params)
{

	GError *error = NULL;
	GVariant *reply = NULL;
	GDBusConnection *connection;

	connection = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, &error);
	if (connection == NULL) {
		_E("failed to get dbus connection");
		return reply;
	}

	reply = g_dbus_connection_call_sync(
			connection,
			dest,
			path,
			interface_name,
			method,
			params,
			NULL,
			G_DBUS_CALL_FLAGS_NONE,
			BM_DN_DBUS_REPLY_TIMEOUT,
			NULL,
			&error);

	g_object_unref(connection);

	if (reply == NULL) {
		if (error != NULL) {
			_E("failed to call dbus method, error [%d: %s]", error->code, error->message);
			g_error_free(error);
		} else {
			_E("failed to call dbus method");
		}

		return NULL;
	}

	return reply;
}

static void __bm_dbus_dictionary_foreach(GVariantIter *iter,
		bm_dbus_dict_cb cb, void *user_data)
{
	char *key = NULL;
	GVariant *value = NULL;

	if (!cb)
		return;

	while (g_variant_iter_loop(iter, "{sv}", &key, &value)) {
		if (key)
			cb(key, value, user_data);
	}
}

static void __bm_extract_app(const char *key, GVariant *value, void *user_data)
{
	app_time_map_st2 *app_data = (app_time_map_st2 *)user_data;

	if (app_data == NULL)
		return;

	if (!g_strcmp0(key, BATTERY_ATM_APP_ID)) {
		const char *app_id = g_variant_get_string(value, NULL);
		app_data->app_id = g_strdup(app_id);
	} else if (!g_strcmp0(key, BATTERY_ATM_RX)) {
		app_data->rx = g_variant_get_uint32(value);
	} else if (!g_strcmp0(key, BATTERY_ATM_TX)) {
		app_data->tx = g_variant_get_uint32(value);
	}
}

static void __bm_extract_dn(const char *key, GVariant *value, void *user_data)
{
	bm_device_network_st *dn_data = (bm_device_network_st *)user_data;

	if (dn_data == NULL)
		return;

	if (!g_strcmp0(key, BATTERY_DN_TIME_LEVEL_0)) {
		dn_data->time_level_0 = g_variant_get_uint32(value);
	} else if (!g_strcmp0(key, BATTERY_DN_TIME_LEVEL_1)) {
		dn_data->time_level_1 = g_variant_get_uint32(value);
	} else if (!g_strcmp0(key, BATTERY_DN_TIME_LEVEL_2)) {
		dn_data->time_level_2 = g_variant_get_uint32(value);
	} else if (!g_strcmp0(key, BATTERY_DN_TIME_LEVEL_3)) {
		dn_data->time_level_3 = g_variant_get_uint32(value);
	} else if (!g_strcmp0(key, BATTERY_DN_TIME_LEVEL_4)) {
		dn_data->time_level_4 = g_variant_get_uint32(value);
	} else if (!g_strcmp0(key, BATTERY_DN_TIME_LEVEL_5)) {
		dn_data->time_level_5 = g_variant_get_uint32(value);
	} else if (!g_strcmp0(key, BATTERY_DN_TIME_LEVEL_6)) {
		dn_data->time_level_6 = g_variant_get_uint32(value);
	} else if (!g_strcmp0(key, BATTERY_ATM_LIST)) {
		GVariant *list_value = NULL;
		GVariantIter *iter = NULL;
		GVariantIter *iter_row = NULL;

		g_variant_get(value, "v", &list_value);
		g_variant_get(list_value, "aa{sv}", &iter);

		while (g_variant_iter_next(iter, "a{sv}", &iter_row)) {
			app_time_map_st2 *app_data = g_try_malloc0(sizeof(app_time_map_st2));
			if (app_data != NULL) {
				memset(app_data, 0, sizeof(app_time_map_st2));

				__bm_dbus_dictionary_foreach(iter_row, __bm_extract_app, app_data);

				dn_data->atm_list = g_slist_append(dn_data->atm_list, app_data);
			}

			g_variant_iter_free(iter_row);
		}

		g_variant_iter_free(iter);
	}
}

static int __bm_dn_get_list(bm_data_h *handle)
{
	GVariant *message = NULL;
	GVariantIter *iter = NULL;
	GVariantIter *iter_row = NULL;
	bm_device_network_st *dn_list = NULL;
	bm_device_network_st *curr_dn_data = NULL;

	message = __invoke_dbus_method(NETCONFIG_SERVICE,
		NETCONFIG_NETWORK_PATH, NETCONFIG_BATTERY_INTERFACE,
		"GetBatteryDnList", NULL);

	if (message == NULL)
		return BM_PLUGIN_ERROR_NO_DATA;

	g_variant_get(message, "(aa{sv})", &iter);

	while (g_variant_iter_next(iter, "a{sv}", &iter_row)) {
		bm_device_network_st *dn_data = g_try_malloc0(sizeof(bm_device_network_st));
		if (dn_data != NULL) {
			memset(dn_data, 0, sizeof(bm_device_network_st));

			__bm_dbus_dictionary_foreach(iter_row, __bm_extract_dn, dn_data);

			if (dn_list == NULL) {
				dn_list = dn_data;
				curr_dn_data = dn_data;
			} else {
				curr_dn_data->dn_data_list = g_slist_append(curr_dn_data->dn_data_list, dn_data);
				curr_dn_data = dn_data;
			}
		}
		g_variant_iter_free(iter_row);
	}

	g_variant_iter_free(iter);
	g_variant_unref(message);

	*handle = dn_list;

	return BM_PLUGIN_ERROR_NONE;
}

static void __bm_dn_print_list(bm_data_h handle)
{
	bm_device_network_st *dn_data = (bm_device_network_st *)handle;

	_D("===============================================================");

	while (dn_data != NULL) {
		GSList *atm_list = NULL;
		GSList *data_list = NULL;

		_D("device network rssi[%d/%d/%d/%d/%d/%d/%d]",
			dn_data->time_level_0, dn_data->time_level_1, dn_data->time_level_2,
			dn_data->time_level_3, dn_data->time_level_4, dn_data->time_level_5,
			dn_data->time_level_6);

		for (atm_list = dn_data->atm_list; atm_list != NULL; atm_list = atm_list->next) {
			app_time_map_st2 *app_data = (app_time_map_st2 *)atm_list->data;
			_D("app_id[%s] rx[%d] tx[%d]", app_data->app_id, app_data->rx, app_data->tx);
		}

		_D("===============================================================");

		if (dn_data->dn_data_list == NULL)
			break;

		data_list = dn_data->dn_data_list;
		dn_data = (bm_device_network_st *)data_list->data;
	}
}

int init(const bm_callbacks_st *callbacks)
{
	ENTER;

	if (!callbacks) {
		_E("invalid input param");
		return BM_PLUGIN_ERROR_INVALID_PARAMETER;
	}

/*	Initialize the necessary paramters and callbacks if required */

	EXIT;
	return BM_PLUGIN_ERROR_NONE;
}

int deinit()
{
	ENTER;

/*	De-initialize callbacks & parameters */

	EXIT;
	return BM_PLUGIN_ERROR_NONE;
}

int get_feature_data(bm_data_h *handle, bm_plugin_data_type_e type)
{
	ENTER;

	int ret = 0;

	ret = __bm_dn_get_list(handle);
	if (ret == 0) {
		__bm_dn_print_list(*handle);
		EXIT;
		return BM_PLUGIN_ERROR_NONE;
	} else {
		EXIT;
		return BM_PLUGIN_ERROR_NO_DATA;
	}
}

static bm_api_st bm_dn_api = {
	.init = init,
	.deinit = deinit,
	.get_feature_data = get_feature_data
};

int plugin_init(bm_api_st ** api)
{
	ENTER;

	*api = &bm_dn_api;

	EXIT;
	return BM_PLUGIN_ERROR_NONE;
}

int plugin_deinit(void)
{
	ENTER;

	EXIT;
	return BM_PLUGIN_ERROR_NONE;
}

BM_PLUGIN_ADD(BM_PLUGIN_ID_DEVICE_NETWORK, NAME, AUTHOR, VERSION, plugin_init, plugin_deinit);
