/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <gio/gio.h>
#include <aul.h>
#include <sys/types.h>
#include <unistd.h>

#include "bm_display_plugin.h"
#include "bm_display_util.h"
#include "bm_log.h"

#define DBUS_DEVICED "org.tizen.system.deviced"
#define DBUS_DEVICED_BM_PATH "/Org/Tizen/System/DeviceD/BatteryMonitor"
#define DBUS_DEVICED_BM_IFACE "org.tizen.system.deviced.BatteryMonitor"
#define DBUS_DEVICED_BM_MEMBER "GetBMData"

static GDBusConnection *dbus_connection;

static int init_dbus(void)
{
	GDBusConnection *conn;
	GError *err = NULL;

	ENTER;

	if (dbus_connection) {
		_D("dbus connection already initialized");
		return 0;
	}

	conn = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, &err);
	if (!conn || err) {
		_E("failed to get dbus connection:%s", err->message);
		g_error_free(err);
		conn = NULL;
		return -1;
	}

	dbus_connection = conn;

	EXIT;

	return 0;
}

static void release_dbus(void)
{
	ENTER;

	if (dbus_connection)
		g_object_unref(dbus_connection);
	dbus_connection = NULL;

	EXIT;
}

static void _atm_list_free(gpointer data)
{
	app_time_map_st1 *atm = (app_time_map_st1 *)data;

	if (!atm) {
		_E("atm is null");
		return;
	}

	if (atm->app_id)
		g_free(atm->app_id);

	free(atm);
}

static void _bds_list_free(gpointer data)
{
	bm_display_st *bds = (bm_display_st *)data;

	if (!bds) {
		_E("bds is null");
		return;
	}

	if (bds->atm_list)
		g_slist_free_full(bds->atm_list, _atm_list_free);

	free(bds);
}

static bm_display_st *_variant_to_bds(GVariant *param)
{
	bm_display_st *bds_header = NULL;
	bm_display_st *bds = NULL;
	GSList *bds_list = NULL;
	app_time_map_st1 *atm = NULL;
	GVariantIter *iter = NULL;
	GVariantIter *sub_iter = NULL;
	GSList *atm_list = NULL;
	guint vals[3];
	guint64 t_start, t_end;
	gchar *app_id = NULL;
	guint app_time = 0;

	ENTER;

	if (!param) {
		_E("Wrong param:null");
		return NULL;
	}

	g_variant_get(param, "(a(uuuxxa(su)))", &iter);
	if (g_variant_iter_n_children(iter) == 0) {
		_E("failed to get data from gvariant. array size is 0.");
		goto err;
	}

	/* make bds list */
	while (g_variant_iter_loop(iter, "(uuuxxa(su))",
					&vals[0], &vals[1], &vals[2], &t_start, &t_end,
					&sub_iter)) {
		/* make atm list */
		atm_list = NULL;
		while (g_variant_iter_loop(sub_iter, "(su)", &app_id, &app_time)) {
			atm = (app_time_map_st1 *)malloc(sizeof(app_time_map_st1));
			if (!atm) {
				_E("failed to allocate memory");
				goto err;
			}

			atm->app_id = g_strdup(app_id);
			atm->time = app_time;

			atm_list = g_slist_prepend(atm_list, atm);
		}

		bds = (bm_display_st *)malloc(sizeof(bm_display_st));
		if (!bds) {
			_E("failed to allocate memory");
			g_slist_free_full(atm_list, _atm_list_free);
			goto err;
		}

		bds->high = vals[0];
		bds->low = vals[1];
		bds->med = vals[2];
		bds->start = t_start;
		bds->stop = t_end;
		bds->atm_list = g_slist_reverse(atm_list);
		bds->display_list = NULL;

		bds_list = g_slist_prepend(bds_list, bds);
	}

	bds_header = (bm_display_st *)calloc(1, sizeof(bm_display_st));
	if (!bds_header) {
		_E("failed to allocate memory");
		goto err;
	}

	bds_header->display_list = g_slist_reverse(bds_list);

	g_variant_iter_free(iter);

	EXIT;

	return bds_header;

err:
	if (iter)
		g_variant_iter_free(iter);
	if (bds_list)
		g_slist_free_full(bds_list, _bds_list_free);

	EXIT;
	return NULL;
}

static bm_display_st *_get_display_info(void)
{
	GVariant *reply = NULL;
	GError *err = NULL;
	bm_display_st *bds = NULL;

	ENTER;

	if (!dbus_connection) {
		_E("dbus connection is null");
		EXIT;
		return NULL;
	}

	reply = g_dbus_connection_call_sync(dbus_connection,
						DBUS_DEVICED,
						DBUS_DEVICED_BM_PATH,
						DBUS_DEVICED_BM_IFACE,
						DBUS_DEVICED_BM_MEMBER,
						NULL, NULL,
						G_DBUS_CALL_FLAGS_NONE,
						-1,
						NULL,
						&err);
	if (!reply || err) {
		if (err) {
			_E("failed to g_dbus_connection_call_sync:%s", err->message);
			g_error_free(err);
		}
		else {
			_E("failed to g_dbus_connection_call_sync");
		}
		return NULL;
	}

	/* convert gvariant into bds list */
	bds = _variant_to_bds(reply);

	g_variant_unref(reply);

	EXIT;

	return bds;
}

int display_init(const bm_callbacks_st *callbacks)
{
	int ret;

	ENTER;

	if (!callbacks) {
		_E("invalid input param");
		return BM_PLUGIN_ERROR_INVALID_PARAMETER;
	}

	ret = init_dbus();
	if (ret)
		return BM_PLUGIN_ERROR_SERVICE_UNAVAILABLE;

	EXIT;
	return BM_PLUGIN_ERROR_NONE;
}

int display_deinit()
{
	ENTER;

	release_dbus();

	EXIT;
	return BM_PLUGIN_ERROR_NONE;
}

int get_display_feature_data(bm_data_h *handle, bm_plugin_data_type_e type)
{
	bm_display_st *bds;

	bds = _get_display_info();
	if (!bds) {
		_E("failed to get bds info from deviced");
		*handle = NULL;
		return BM_PLUGIN_ERROR_NO_DATA;
	}

	*handle = bds;

	return BM_PLUGIN_ERROR_NONE;
}

static bm_api_st display_api = {
	.init = display_init,
	.deinit = display_deinit,
	.get_feature_data = get_display_feature_data
};

int plugin_init(bm_api_st ** api)
{
	ENTER;

	*api = &display_api;

	EXIT;
	return 0;
}

int plugin_deinit(void)
{
	ENTER;

	EXIT;
	return 0;
}

BM_PLUGIN_ADD(BM_PLUGIN_ID_DISPLAY, NAME, AUTHOR, VERSION, plugin_init, plugin_deinit);
